{% extends 'timeseer.html' %}

{% from 'macros.html' import menu %}

{% macro help_menu(active) %}
{{
    menu(
        'Help',
        [
            ('Getting started', url_for('help.display_help', sourcename=source_name)),
            ('Admin documentation', url_for('help.display_admin_help', sourcename=source_name)),
            ('Source documentation', url_for('sources.display_documentation_index', sourcename=source_name)),
            ('Score documentation', url_for('help.display_score_help', sourcename=source_name)),
            ('Compliance report', url_for('help.compliance_report', sourcename=source_name)),
        ],
        active,
    )
}}
{% endmacro %}

{% block menu %}
{{ help_menu('Getting started') }}
{% endblock %}

{% block main %}
<div class="ts-section py-3 my-3">
    <h2 class="h4">Getting started</h2>
</div>

<div class="my-3 py-3">
    <a class="btn btn-primary" href="mailto:feedback@timeseer.ai">Make a wish</a>
    <a class="btn btn-primary" href="mailto:bugs@timeseer.ai">Report a bug</a>
</div>
{% endblock %}
