{% set multivariate_title = 'Statistics' %}
{% extends 'multivariate-layout.html' %}

{% block styles %}
<style>
    .score-table {
        table-layout: fixed;
    }
    .score-table .score-grouping {
        width: 20%;
    }
    .score-table .score {
        width: 20%;
    }
</style>
{% endblock %}

{% block menu %}
{{ multivariate_menu(multivariate_title) }}
{% endblock %}

{% block multivariate_content %}
{% if statistics|count > 0 %}
{% include 'fragments/statistics-table.html' %}
{% else %}
<div class="alert alert-info">
    No statistics available.
</div>
{% endif %}
{% endblock %}
